'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

/**
 * Returns whether a string is a percentage.
 *
 * @param   {String} str
 * @returns {Boolean}
 */
var isPercentage = function isPercentage(str) {
	return (/^\d+(\.\d+)?%/g.test(str)
	);
};

/**
 * Determines if a cue can be triggered.
 *
 * @param   {Object} cue
 * @returns {Boolean}
 */
var canTrigger = function canTrigger(cue) {
	return cue.enabled && !cue.triggered;
};

/**
 * Parses the time offset of a cue.
 *
 * @param   {Number || String} offset
 * @param   {Number}           duration
 * @returns {Number}
 */
var parseTime = function parseTime(offset, duration) {

	if (isPercentage(offset)) {

		return parseFloat(offset) / 100 * duration;
	} else {

		return parseFloat(offset);
	}
};

/**
 * Attempts to trigger a cue point based on the current time of a video element.
 *
 * @type {Function} (video:HTMLVideoElement) => (cue:Object) => void
 */
var tryTrigger = function tryTrigger(video) {
	return function (cue) {

		var time = parseTime(cue.offset, video.duration);

		if (video.currentTime >= time) {

			cue.triggered = true;
			setTimeout(function () {
				return cue.handler({ offset: cue.offset });
			}, 0);
		}
	};
};

/**
 * Creates a new cue list.
 *
 * @param   {HTMLVideoElement} video    The video element to trigger cue points.
 * @returns {Object}
 */
function createCueList(video) {

	var cueList = [];
	var trigger = function trigger() {
		return cueList.filter(canTrigger).forEach(tryTrigger(video));
	};

	video.addEventListener('playing', trigger);
	video.addEventListener('timeupdate', trigger);

	return {

		/**
   * Adds cue points to the list.
   *
   * @param {Array}    offsets    Values representing the progress required to trigger a cue point.
   *                              Either a number of seconds, or a percentage as a string.
   * @param {Function} handler    The function to call when a cue point is triggered.
   */
		add: function add(offsets, handler) {

			offsets = Array.isArray(offsets) ? offsets : [offsets];
			offsets.map(function (offset) {
				cueList.push({
					offset: offset,
					handler: handler,
					triggered: false,
					enabled: true
				});
			});
		},


		/**
   * Removes cue points with the given offsets. All cue points are removed if no argument is passed.
   *
   * @param {Array} offsets   A list of offsets as a number of seconds, or a percentage as a string.
   */
		remove: function remove(offsets) {

			if (offsets === undefined) {

				cueList = [];
			} else {

				offsets = Array.isArray(offsets) ? offsets : [offsets];
				cueList = cueList.filter(function (cue) {
					return offsets.indexOf(cue.offset) === -1;
				});
			}
		},


		/**
   * Gets the offsets of all cue points.
   *
   * @returns {Array}
   */
		offsets: function offsets() {

			return cueList.map(function (cue) {
				return cue.offset;
			});
		},


		/**
   * Disables all cue points.
   */
		disable: function disable() {

			cueList.forEach(function (cue) {
				cue.enabled = false;
			});
		},


		/**
   * Enables all cue points.
   */
		enable: function enable() {

			cueList.forEach(function (cue) {
				cue.enabled = true;
			});
		},


		/**
   * Resets the triggered state of all cue points.
   */
		reset: function reset() {

			cueList.forEach(function (cue) {
				cue.triggered = false;
			});
		},


		/**
   * Clears the the cue list and removes all event listeners.
   */
		dispose: function dispose() {

			cueList = [];
			video.removeEventListener('playing', trigger);
			video.removeEventListener('timeupdate', trigger);
		}
	};
}

var cueList = { create: createCueList };

exports.CueList = cueList;
